@@ -22,7 +22,6 @@
 #ifndef EPHY_EMBED_H
 #define EPHY_EMBED_H
 
-#include <glib.h>
 #include <glib-object.h>
 #include <gtk/gtk.h>
 
@@ -37,22 +36,26 @@ G_BEGIN_DECLS
 #define EPHY_IS_EMBED_CLASS(k)        (G_TYPE_CHECK_CLASS_TYPE ((k), EPHY_TYPE_EMBED))
 #define EPHY_EMBED_GET_CLASS(o)       (G_TYPE_INSTANCE_GET_CLASS ((o), EPHY_TYPE_EMBED, EphyEmbedClass))
 
-typedef struct EphyEmbedClass EphyEmbedClass;
-typedef struct EphyEmbed EphyEmbed;
-typedef struct EphyEmbedPrivate EphyEmbedPrivate;
+typedef struct _EphyEmbedClass EphyEmbedClass;
+typedef struct _EphyEmbed EphyEmbed;
+typedef struct _EphyEmbedPrivate EphyEmbedPrivate;
 
-struct EphyEmbed {
-  GtkBin parent_instance;
+struct _EphyEmbed {
+  GtkVBox parent_instance;
 
   /*< private >*/
   EphyEmbedPrivate *priv;
 };
 
-struct EphyEmbedClass {
-  GtkBinClass parent_class;
+struct _EphyEmbedClass {
+  GtkVBoxClass parent_class;
 };
 
-GType            ephy_embed_get_type (void);
+GType        ephy_embed_get_type          (void);
+EphyWebView* ephy_embed_get_web_view      (EphyEmbed *embed);
+void         ephy_embed_add_top_widget    (EphyEmbed *embed, GtkWidget *widget, gboolean destroy_on_transition);
+void         ephy_embed_remove_top_widget (EphyEmbed *embed, GtkWidget *widget);
+void         ephy_embed_auto_download_url (EphyEmbed *embed, const char *url);
 
 G_END_DECLS
 
