@@ -20,19 +20,19 @@
 
 #include "config.h"
 
+#include "ephy-settings.h"
 #include "ephy-shell.h"
 #include "ephy-file-helpers.h"
 #include "ephy-object-helpers.h"
 #include "ephy-state.h"
 #include "ephy-debug.h"
 #include "ephy-stock-icons.h"
-#include "eel-gconf-extensions.h"
 #include "ephy-dbus-client-bindings.h"
 #include "ephy-activation.h"
 #include "ephy-session.h"
 #include "ephy-shell.h"
 #include "ephy-prefs.h"
-#include "ephy-profile-migration.h"
+#include "ephy-profile-utils.h"
 #include "ephy-debug.h"
 #include "eggsmclient.h"
 
@@ -50,10 +50,6 @@
 #include <girepository.h>
 #endif
 
-#ifdef HAVE_LIBNOTIFY
-#include <libnotify/notify.h>
-#endif
-
 static GQuark startup_error_quark = 0;
 #define STARTUP_ERROR_QUARK	(startup_error_quark)
 
@@ -202,41 +198,6 @@ slowly_and_stupidly_obtain_timestamp (Display *xdisplay)
 	return event.xproperty.time;
 }
 
-static void
-handle_url (GtkAboutDialog *about,
-	    const char *link,
-	    gpointer data)
-{
-	ephy_shell_new_tab (ephy_shell_get_default (),
-			    NULL, NULL, link,
-			    EPHY_NEW_TAB_OPEN_PAGE);
-}
-
-static void
-handle_email (GtkAboutDialog *about,
-	      const char *link,
-	      gpointer data)
-{
-	char *command, *handler;
-	GAppInfo *appinfo;
-
-	if (eel_gconf_get_boolean ("/desktop/gnome/url-handlers/mailto/enabled") == FALSE)
-	{
-		return;
-	}
-	/* FIXME: better use g_app_info_get_default_for_uri_scheme () when it is
-	 * implemented.
-	 */
-	handler = eel_gconf_get_string ("/desktop/gnome/url-handlers/mailto/command");
-	command = g_strconcat (handler, "mailto:", link, NULL);
-	appinfo = g_app_info_create_from_commandline (command, NULL, 0, NULL);
-	ephy_file_launch_application (appinfo, NULL,
-				      gtk_get_current_event_time (),
-				      GTK_WIDGET (about));
-	g_free (handler);
-	g_free (command);
-}
-
 static void
 unref_proxy_reply_cb (DBusGProxy *proxy,
 		      GError *error,
@@ -473,6 +434,7 @@ main (int argc,
 	DBusGProxy *proxy;
 	GError *error = NULL;
 	guint32 user_time;
+	gboolean arbitrary_url;
 
 #ifdef ENABLE_NLS
 	/* Initialize the i18n stuff */
@@ -490,6 +452,9 @@ main (int argc,
 	 */
 	LIBXML_TEST_VERSION;
 
+	/* sets name to help matching with the .desktop file */
+	g_set_prgname ("epiphany");
+
 	/* If we're given -remote arguments, translate them */
 	if (argc >= 2 &&
 	    strcmp (argv[1], "-remote") == 0)
@@ -604,8 +569,10 @@ main (int argc,
 		exit (1);
 	}
 
-	if (arguments != NULL &&
-	    eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_ARBITRARY_URL))
+	arbitrary_url = g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
+						EPHY_PREFS_LOCKDOWN_ARBITRARY_URL);
+
+	if (arguments != NULL && arbitrary_url)
 	{
 		g_print ("URL loading is locked down\n");
 		exit (1);
@@ -673,7 +640,7 @@ main (int argc,
 	/* Get a timestamp manually if need be */
 	if (user_time == 0)
 	{
-		user_time = slowly_and_stupidly_obtain_timestamp (gdk_display);
+		user_time = slowly_and_stupidly_obtain_timestamp (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
 	}
 
 	/* sets the name to appear in the window list applet when grouping windows */
@@ -747,17 +714,27 @@ main (int argc,
 	}
 
 	/* Migrate profile if we are not running a private instance */
-        if (ephy_has_private_profile () == FALSE)
-          _ephy_profile_migrate ();
+	if (ephy_has_private_profile () == FALSE &&
+	    ephy_profile_utils_get_migration_version () < EPHY_PROFILE_MIGRATION_VERSION)
+	{
+		GError *error = NULL;
+		char *argv[1] = { "ephy-profile-migrator" };
+		char *envp[1] = { "EPHY_LOG_MODULES=ephy-profile" };
+
+		g_spawn_sync (NULL, argv, envp, G_SPAWN_SEARCH_PATH,
+			      NULL, NULL, NULL, NULL,
+			      NULL, &error);
+
+		if (error)
+		{
+			LOG ("Failed to run migrator: %s", error->message);
+			g_error_free (error);
+		}
+	}
 
-	eel_gconf_monitor_add ("/apps/epiphany/general");
 	ephy_stock_icons_init ();
 	load_accels ();
 
-	/* Extensions may want these, so don't initialize in window-cmds */
-	gtk_about_dialog_set_url_hook (handle_url, NULL, NULL);
-	gtk_about_dialog_set_email_hook (handle_email, NULL, NULL);
-
 	/* Work-around Flash Player crash */
 	g_setenv ("XLIB_SKIP_ARGB_VISUALS", "1", FALSE);
 
@@ -767,23 +744,14 @@ main (int argc,
 
 	queue_commands (user_time);
 
-#ifdef HAVE_LIBNOTIFY	
-	/* Init notifications for the download manager */
-	notify_init (PACKAGE);
-#endif
-
 	gtk_main ();
 
 	/* Shutdown */
 	g_object_unref (ephy_shell);
 
-#ifdef HAVE_LIBNOTIFY	
-	if (notify_is_initted ())
-		notify_uninit ();
-#endif
-	eel_gconf_monitor_remove ("/apps/epiphany/general");
 	save_accels ();
 	ephy_state_save ();
+	ephy_settings_shutdown ();
 	ephy_file_helpers_shutdown ();
 	xmlCleanupParser ();
 
